;------------------------------- ANIMATE CLEANUP -------------------------------
pro Animate_cleanup,wTopBase
widget_control,wTopBase,get_uvalue=State
obj_destroy,State.oHolder
obj_destroy,State.oWindow
if widget_info(State.wParamBase,/valid_id) then widget_control,State.wParamBase,/destroy
if widget_info(State.wExampleBase,/valid_id) then widget_control,State.wExampleBase,/destroy
if widget_info(State.wWaitBase,/valid_id) then widget_control,State.wWaitBase,/destroy
if widget_info(State.wHelpBase,/valid_id) then widget_control,State.wHelpBase,/destroy
!except=1
end

;------------------------------- TEXTURE CLEANUP -------------------------------
pro Texture_cleanup,wTextureBase
widget_control,wTextureBase,get_uvalue=State
widget_control,State.wTopBase,sensitive=1
if widget_info(State.wParamBase,/valid_id) then widget_control,State.wParamBase,sensitive=1
end

;------------------------------- OBJECT CLEANUP -------------------------------
pro Object_cleanup,wObjectBase
widget_control,wObjectBase,get_uvalue=State
widget_control,State.wTopBase,sensitive=1
if widget_info(State.wParamBase,/valid_id) then widget_control,State.wParamBase,sensitive=1
end

;------------------------------- SAVE CLEANUP -------------------------------
pro Save_cleanup,wSaveBase
widget_control,wSaveBase,get_uvalue=State
widget_control,State.wTopBase,sensitive=1
if widget_info(State.wParamBase,/valid_id) then widget_control,State.wParamBase,sensitive=1
end

;--------------------------------REDRAW-----------------------------------------
pro redraw,xdim,ydim,draw,winerase=winerase,Condition=condition
widget_control,draw,get_value=window1 & wset,window1
if keyword_set(winerase) then erase
if keyword_set(condition) then begin
   widget_control,Condition,get_value=Val & Val=Val(0)
   if Val eq 'ERASE CENTRAL LINE' then fulfilled=1 else fulfilled=0
endif else fulfilled=1
if fulfilled eq 1 then begin
   device,get_graphics=old
   device,set_graphics=6
   arrow,xdim/2,0,xdim/2,ydim,hsize=0,color=!D.N_colors-1
   device,set_graphics=old
endif
end

;------------------------------- ANGLES -----------------------------------
function Get_angles,axis
theta=round(acos(axis(2))*!radeg) ;<0,pi>
if (theta eq 0) or (theta eq 180) then fi=0 else begin
   if axis(0) gt 0 then begin
      if axis(1) gt 0 then fi=atan(axis(1)/axis(0))*!radeg
      if axis(1) lt 0 then fi=atan(axis(1)/axis(0))*!radeg+360
      if axis(1) eq 0 then fi=0
   endif
   if axis(0) lt 0 then begin
      if axis(1) gt 0 then fi=atan(axis(1)/axis(0))*!radeg+180
      if axis(1) lt 0 then fi=atan(axis(1)/axis(0))*!radeg+180
      if axis(1) eq 0 then fi=180
   endif
   if axis(0) eq 0 then begin
      if axis(1) gt 0 then fi=90
      if axis(1) lt 0 then fi=270
      if axis(1) eq 0 then fi=0
   endif
endelse
fi=round(fi)
if fi eq 360 then fi=0
thetatext=strcompress(string(theta),/remove_all)
fitext=strcompress(string(fi),/remove_all)
return,'Angles '+thetatext+' '+fitext
end

;----------------------------- COORDINATES -------------------------------
function Make_coord,State
   if (State.ActualObject eq 'cone') or (State.ActualObject eq 'ellipsoid') then begin
      nlevels=State.density
      nfi=4*(State.density-1)
      n=(nlevels-1)*nfi+1
      limit=nlevels-2
   endif
   if (State.ActualObject eq 'spiral') then begin
      nfi=State.density
      nlevels=fix(State.density*State.Height*7)
      n=nfi*nlevels
      limit=nlevels-1
   endif
   if (State.ActualObject eq 'cylinder') then begin
      nfi=4*(State.density-1)
      nlevels=State.density
      n=nlevels*nfi
      limit=nlevels-1
   endif

   coordinates=fltarr(2,n)
   xcoord=findgen(nfi)*2.0/nfi
   xcoord(nfi/2+1:nfi-1)=reverse(findgen(nfi-(nfi/2+1))*2.0/nfi+2.0/nfi)
   index=0
   for i=0,limit do begin
    for j=0,nfi-1 do begin
      coordinates(1,index)=State.up-float(i)*(State.up-State.down)/(nlevels-1)
      coordinates(0,index)=xcoord(j)
      index=index+1
    endfor
   endfor

   if (State.ActualObject eq 'cone') or (State.ActualObject eq 'ellipsoid') then begin
      coordinates(1,index)=State.down
      coordinates(0,index)=0.5
   endif

return,coordinates
end

;----------------------------- BOTTOM COORDINATES -------------------------------
function Make_bottom_coord,State
   if (State.ActualObject eq 'cone') then begin
      nlevels=fix(State.density*State.Radius*State.Distribution/sqrt((State.Radius*State.Distribution)^2+$
              (State.Height)^2))
      if (nlevels lt 3) then nlevels=3
      nfi=4*(State.density-1)
      n=nlevels*nfi
      limit=nlevels-2
   endif
   if (State.ActualObject eq 'ellipsoid') then begin
      b=State.a*sqrt(1-(State.exc)^2)
      nlevels=fix(State.density*b*State.Distribution/sqrt((b*State.Distribution)^2+(State.a)^2))
      if (nlevels lt 3) then nlevels=3
      nfi=4*(State.density-1)
      n=nlevels*nfi
      limit=nlevels-2
   endif
   if (State.ActualObject eq 'spiral') then begin
      nlevels=3
      nfi=State.density
      n=(nlevels-1)*nfi+1
      limit=nlevels-2
   endif
   if (State.ActualObject eq 'cylinder') then begin
      nlevels=fix(State.density*State.Radius*State.Distribution/State.Height)
      if (nlevels lt 3) then nlevels=3
      nfi=4*(State.density-1)
      n=nlevels*nfi
      limit=nlevels-2
   endif

   coordinates=fltarr(2,n)
   xcoord=findgen(nfi)*2.0/nfi
   xcoord(nfi/2+1:nfi-1)=reverse(findgen(nfi-(nfi/2+1))*2.0/nfi+2.0/nfi)
   index=0
   for i=0,limit do begin
    for j=0,nfi-1 do begin
      coordinates(1,index)=State.up_bottom-float(i)*(State.up_bottom-State.down_bottom)/(nlevels-1)
      coordinates(0,index)=xcoord(j)
      index=index+1
    endfor
   endfor

  if (State.ActualObject eq 'spiral') then begin
      coordinates(1,index)=State.down_bottom
      coordinates(0,index)=0.5
  endif

return,coordinates
end
;----------------------------- PARAM EVENTS -------------------------------
pro Param_event,event
widget_control,event.id,get_uvalue=uvalue
widget_control,event.top,get_uvalue=State
widget_control,State.wTopBase,get_uvalue=State

case uvalue of
'VEL1':begin ;rychlost rotace
       widget_control,event.id,get_value=Val
       State.ParamRot=Val
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'VEL2':begin ;rychlost po plasti telesa
       widget_control,event.id,get_value=Val
       State.ParamTex=Val
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'VEL3':begin ;rychlost po spirale
       widget_control,event.id,get_value=Val
       State.V=Val
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'PAR1':begin ;radius (kuzel,valec)
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.Radius=Val
       State.oObject->SetProperty,radius=Val
       State.oBottom->SetProperty,radius=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       State.oInner->SetProperty,radius=Val-Val*State.Distribution
       State.oInner->SetProperty,texture_coord=Make_coord(State)
       if (State.ActualObject eq 'cylinder') then begin
          State.oBottom2->SetProperty,radius=Val
          State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       endif
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'PAR2':begin ;height (kuzel,valec,spirala)
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.Height=Val
       State.oObject->SetProperty,height=Val
       State.oBottom->SetProperty,height=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       if (State.ActualObject eq 'cylinder') then begin
           State.oBottom2->SetProperty,height=Val
           State.oBottom2->SetProperty,pos=[0.0,0.0,Val]
           State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       endif
       if (State.ActualObject eq 'spiral') then begin
          State.oBottom2->SetProperty,height=Val
          State.oBottom2->SetProperty,pos=[0.0,0.0,Val]
          State.oObject->SetProperty,texture_coord=Make_coord(State)
          State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
          State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       endif else begin
          State.oInner->SetProperty,height=Val
          State.oInner->SetProperty,texture_coord=Make_coord(State)
       endelse
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'PAR3':begin  ;exc (elipsoid)
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.Exc=Val
       State.oObject->SetProperty,exc=Val
       State.oBottom->SetProperty,exc=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       b=State.A*sqrt(1-(State.Exc)^2)
       exc2=sqrt(1-(b-b*State.Distribution)^2/(State.A)^2)
       State.oInner->SetProperty,exc=exc2
       State.oInner->SetProperty,texture_coord=Make_coord(State)
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'PAR4':begin ;a (elipsoid)
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.A=Val
       State.oObject->SetProperty,a=Val
       State.oBottom->SetProperty,a=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       b=State.A*sqrt(1-(State.Exc)^2)
       exc2=sqrt(1-(b-b*State.Distribution)^2/(State.A)^2)
       State.oInner->SetProperty,exc=exc2,a=State.A
       State.oInner->SetProperty,texture_coord=Make_coord(State)
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'PAR5':begin ;width (spirala)
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.Width=Val
       State.oObject->SetProperty,width=Val
       State.oBottom->SetProperty,width=Val
       State.oBottom2->SetProperty,width=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       if (Val eq 0.0) then State.oObject->SetProperty,style=1 else State.oObject->SetProperty,style=2
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'PAR6':begin ;r (spirala)
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.R=Val
       State.oObject->SetProperty,r=Val
       State.oBottom->SetProperty,r=Val
       State.oBottom2->SetProperty,r=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'PAR8':begin ;dist (spirala)
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.Dist=Val
       State.oObject->SetProperty,dist=Val
       State.oBottom->SetProperty,dist=Val
       State.oBottom2->SetProperty,dist=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'QUA1':begin ;kvalita zobrazeni
       widget_control,event.id,get_value=Val
       Val=Val+3 ;na slideru od 0 do 15, jako parametr objektu od 3 do 18
       State.density=Val
       State.oObject->SetProperty,density=Val
       State.oObject->SetProperty,texture_coord=Make_coord(State)
       State.oBottom->SetProperty,density=Val
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
       if (State.ActualObject eq 'cylinder') or (State.ActualObject eq 'spiral') then begin
          State.oBottom2->SetProperty,density=Val
          State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       endif
       if (State.ActualObject ne 'spiral') then begin
          State.oInner->SetProperty,density=Val
          State.oInner->SetProperty,texture_coord=Make_coord(State)
       endif
       end
'QUA2':begin ;jemnost deleni objemu
       widget_control,event.id,get_value=Val
       State.step=Val/100.0
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'QUA3':begin ;jemnost deleni sterbiny
       widget_control,event.id,get_value=Val
       State.Nstep=Val
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'SLIT1':begin ;pozice sterbiny
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.SlitPos=Val
       State.oSlit1->SetProperty,pos=Val
       State.oSlit2->SetProperty,pos=Val+State.SlitWid-0.01
       State.oWindow->Draw,State.oView
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'SLIT2':begin ;sirka sterbiny
       widget_control,event.id,get_value=Val
       Val=Val/100.0
       State.SlitWid=Val
       State.oSlit2->SetProperty,pos=State.SlitPos+Val-0.01
       State.oWindow->Draw,State.oView
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'LINE':begin ;sirka spektralni cary
       widget_control,event.id,get_value=Val
       State.LineWidth=Val
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'NUM1':begin ;jemnost deleni spiraly pri vypoctu pruseciku
       widget_control,event.id,get_value=Val
       State.Levels=Val
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'NUM2':begin ;presnost vypoctu pruseciku se spiralou
       widget_control,event.id,get_value=Val
       State.Accuracy=Val
       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end
'BUT1':begin ;stredni cara v pravem okne
       widget_control,event.id,get_value=Val & Val=Val(0)
       if Val eq 'SHOW CENTRAL LINE' then begin
          widget_control,event.id,set_button=1
          widget_control,event.id,set_value='ERASE CENTRAL LINE'
       endif else begin
          widget_control,event.id,set_button=0
          widget_control,event.id,set_value='SHOW CENTRAL LINE'
       endelse
       redraw,State.xdim,State.ydim,State.wRightDraw
       end
'DIST':begin ;rozlozeni hmoty
       widget_control,event.id,get_value=Val & Val=Val(0)
       State.Distribution=Val/100.0
       if Val eq 0 then widget_control,State.wParamSlider,sensitive=0 $
                   else widget_control,State.wParamSlider,sensitive=1

       State.oBottom->SetProperty,inner=State.Distribution
       State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)

       if State.ActualObject eq 'ellipsoid' then begin
         b=State.A*sqrt(1-(State.Exc)^2)
         exc2=sqrt(1-(b-b*State.Distribution)^2/(State.A)^2)
         State.oInner->SetProperty,exc=exc2
       endif else State.oInner->SetProperty,radius=State.Radius-State.Radius*State.Distribution
       State.oInner->SetProperty,texture_coord=Make_coord(State)

       State.oBottom->SetProperty,hide=0
       State.oInner->SetProperty,hide=0
       if (State.ActualObject eq 'cylinder') then begin
          State.oBottom2->SetProperty,inner=State.Distribution
          State.oBottom2->SetProperty,hide=0
          State.oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
       endif

       if State.Distribution eq 0.0 then begin
          State.oBottom->SetProperty,hide=1
          State.oInner->SetProperty,hide=1
          if (State.ActualObject eq 'cylinder') then State.oBottom2->SetProperty,hide=1
       endif

       if State.Distribution eq 1.0 then State.oInner->SetProperty,hide=1

       if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
       end

 'GEN':Generate,State
endcase

widget_control,event.top,set_uvalue=State
widget_control,State.wTopBase,set_uvalue=State
end

;----------------------------- TEXTURE EVENTS -------------------------------
pro Texture_event,event
widget_control,event.id,get_uvalue=uvalue
widget_control,event.top,get_uvalue=State
case uvalue of

'DRAW1':begin
        widget_control,State.wTextureLabel,set_value=State.TexturePath(0)
        end

'DRAW2':begin
        widget_control,State.wTextureLabel,set_value=State.TexturePath(1)
        end

'DRAW3':begin
        widget_control,State.wTextureLabel,set_value=State.TexturePath(2)
        end

'OK':begin
     widget_control,State.wTextureLabel,get_value=val
     State.ActualTexture=val(0)
     widget_control,event.top,/destroy
     if (State.RunAnim ne 0) then begin
         read_jpeg,State.ActualTexture,image
         State.oImage->SetProperty,data=image
         State.oImagespiral->SetProperty,data=image(*,0:50,*)
         if (State.ActualObject ne 'spiral') then State.oObject->SetProperty,texture_map=State.oImage $
                                             else State.oObject->SetProperty,texture_map=State.oImagespiral
         State.oWindow->Draw,State.oView
     endif
     end

'CANCEL':begin
         widget_control,event.top,/destroy
         end

endcase
if (widget_info(event.top,/valid_id) eq 1) then widget_control,event.top,set_uvalue=State
widget_control,State.wTopBase,set_uvalue=State
end

;----------------------------- OBJECT EVENTS -------------------------------
pro Object_event,event
widget_control,event.id,get_uvalue=uvalue
widget_control,event.top,get_uvalue=State

case uvalue of

'DRAW1':begin
        widget_control,State.wObjectLabel,set_value='cone'
        end

'DRAW2':begin
        widget_control,State.wObjectLabel,set_value='ellipsoid'
        end

'DRAW3':begin
        widget_control,State.wObjectLabel,set_value='spiral'
        end

'DRAW4':begin
        widget_control,State.wObjectLabel,set_value='cylinder'
        end

'OK':begin
     widget_control,State.wObjectLabel,get_value=val
     State.ActualObject=val(0)
     widget_control,event.top,/destroy
     if (widget_info(State.wExampleBase,/valid_id)) then widget_control,State.wExampleBase,/destroy
     if (State.ParamExist eq 1) then widget_control,State.wParamBase,/destroy

     ;------------------widgets------------------
     widget_control,State.wInnerBase,/destroy
     State.wInnerBase=widget_base(State.wTopBase,/row)
         State.wLeftBase=widget_base(State.wInnerBase,uvalue='TIMER')
             State.wDraw=widget_draw(State.wLeftBase,xsize=State.xdim,ysize=State.ydim,$
                                   graphics_level=2,retain=0,/button_events,$
                                   /expose_events,uvalue='DRAW')
         State.wRightBase=widget_base(State.wInnerBase)
             State.wRightDraw=widget_draw(State.wRightBase,xsize=State.xdim,ysize=State.ydim,$
                                          /button_events,/motion_events,uvalue='RIGHT')
     widget_control,State.wRightDraw,get_value=Window1
     wset,Window1
     erase
     State.RightDraw=Window1

     ;-----------------objects-------------------
     obj_destroy,State.oHolder
     obj_destroy,State.oWindow

     ;oImage
     read_jpeg,State.ActualTexture,image
     oImage=obj_new('IDLgrImage',image)
     oImagespiral=obj_new('IDLgrImage',image(*,0:50,*))

     ;oObject
     if (State.ActualObject eq 'spiral') then State.Distribution=1.0 else State.Distribution=0.0
     State.Negativ=0

     if (State.ActualObject eq 'cone') then begin
                                oObject=obj_new('cone',color=[150,150,150],radius=State.Radius,height=State.Height,$
                                    texture_map=oImage,bottom=[215,215,215],density=State.density)
                                oBottom=obj_new('cone_bottom',color=[255,255,255],radius=State.Radius,$
                                    height=State.Height,texture_map=oImage,bottom=[255,255,255],density=State.density,$
                                    inner=State.Distribution)
                                oInner=obj_new('cone',color=[150,150,150],radius=State.Radius-State.Radius*State.Distribution, $
                                    height=State.Height,texture_map=oImage,bottom=[215,215,215],density=State.density)
                                endif
     if (State.ActualObject eq 'ellipsoid') then begin
                                oObject=obj_new('ellipsoid',color=[150,150,150],texture_map=oImage,$
                                    bottom=[215,215,215],density=State.density,exc=State.Exc,a=state.A)
                                oBottom=obj_new('ellipsoid_bottom',color=[255,255,255],texture_map=oImage,$
                                    bottom=[255,255,255],density=State.density,exc=State.Exc,a=state.A,$
                                    inner=State.Distribution)
                                b=State.A*sqrt(1-(State.Exc)^2)
                                exc2=sqrt(1-(b-b*State.Distribution)^2/(State.A)^2)
                                oInner=obj_new('ellipsoid',color=[150,150,150],texture_map=oImage,$
                                    bottom=[215,215,215],density=State.density,exc=exc2,a=state.A)
                                endif
     if (State.Width eq 0.0) then spiralstyle=1 else spiralstyle=2
     if (State.ActualObject eq 'spiral') then begin
                                oObject=obj_new('spiral',color=[150,150,150],bottom=[215,215,215],$
                                   texture_map=oImagespiral,density=State.density,height=State.Height,width=State.Width,$
                                   r=State.R,dist=State.Dist,style=spiralstyle)
                                oBottom=obj_new('spiral_bottom',color=[255,255,255],bottom=[255,255,255],$
                                   texture_map=oImagespiral,density=State.density,height=State.Height,width=State.Width,$
                                   r=State.R,dist=State.Dist,style=spiralstyle)
                                oBottom2=obj_new('spiral_bottom',color=[255,255,255],bottom=[255,255,255],$
                                   texture_map=oImagespiral,density=State.density,height=State.Height,width=State.Width,$
                                   r=State.R,dist=State.Dist,style=spiralstyle,pos=[0.0,0.0,State.Height])
                                endif
     if (State.ActualObject eq 'cylinder') then begin
                                oObject=obj_new('cylinder',color=[150,150,150],bottom=[215,215,215],$
                                   texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius)
                                oBottom=obj_new('cylinder_bottom',color=[255,255,255],bottom=[255,255,255],$
                                   texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius,$
                                   inner=State.Distribution)
                                oBottom2=obj_new('cylinder_bottom',color=[255,255,255],bottom=[255,255,255],$
                                   texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius,$
                                   pos=[0.0,0.0,State.Height],inner=State.Distribution)
                                oInner=obj_new('cylinder',color=[150,150,150],bottom=[215,215,215],$
                                   texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius-$
                                   State.Radius*State.Distribution)
                                endif

     ;oSlit
     oSlit1=obj_new('line',pos=-1.0)
     oSlit2=obj_new('line',pos=-1.0)

     ;texture
     oObject->SetProperty,texture_coord=Make_coord(State)
     oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
     if (State.ActualObject ne 'spiral') then begin
        if State.Distribution eq 0.0 then oBottom->SetProperty,hide=1
        oInner->SetProperty,texture_coord=Make_coord(State)
        if State.Distribution eq 0.0 then oInner->SetProperty,hide=1
     endif
     if (State.ActualObject eq 'cylinder') then begin
        oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
        if State.Distribution eq 0.0 then oBottom2->SetProperty,hide=1
     endif

     ;lights
     oLight1=obj_new('IDLgrLight',type=0,intensity=1)
     oLight=obj_new('IDLgrModel')
     oLight->Add,oLight1

     ;oModel
     oModel=obj_new('IDLgrModel')
     oModel->Add,oObject
     oModel->Add,oBottom
     if (State.ActualObject eq 'cylinder') or (State.ActualObject eq 'spiral') then oModel->Add,oBottom2
     if (State.ActualObject ne 'spiral') then oModel->Add,oInner
     oModel->Add,oLight
     RotAxis=[0.0,0.0,1.0]
     oModel->Rotate,[1,0,0],-70
     oModel->GetProperty,transform=Matrix
     RotAxis=RotAxis # Matrix(0:2,0:2)

     ;oView
     oView=obj_new('IDLgrView',color=[0,0,0],zclip=[2,-2])
     oView->Add,oModel
     oView->Add,oSlit1
     oView->Add,oSlit2

     ;oWindow
     widget_control,State.wDraw,get_value=oWindow
     oWindow->Draw,oView

     ;oTrack
     oTrack=obj_new('trackball',[State.xdim/2.0,State.ydim/2.0],State.xdim/2.0)

     ;container
     oHolder=obj_new('IDLgrContainer')
     oHolder->Add,oImage
     oHolder->Add,oImagespiral
     oHolder->Add,oView
     oHolder->Add,oTrack

     ;State
     State.Matrix=Matrix
     State.RotAxis=RotAxis
     State.ParamRot=0L
     State.ParamTex=0L
     State.V=0
     State.oImage=oImage
     State.oImagespiral=oImagespiral
     State.oObject=oObject
     State.oBottom=oBottom
     State.oModel=oModel
     State.oLight=oLight
     State.oView=oView
     State.oHolder=oHolder
     State.oWindow=oWindow
     State.oTrack=oTrack
     State.oSlit1=oSlit1
     State.oSlit2=oSlit2
     State.RunAnim=1
     State.SlitPos=-1.0
     State.SlitWid=0.01
     State.ParamOffset=[0.0,0.0]
     if (State.ActualObject ne 'spiral') then State.oInner=oInner
     if (State.ActualObject eq 'cylinder') or (State.ActualObject eq 'spiral') then State.oBottom2=oBottom2

     ;---------------Parameters Base------------------------
     widget_control,State.wTopBase,tlb_get_offset=offset
     if (State.ActualObject eq 'spiral') then paramx=570 else paramx=465
     wParamBase=widget_base(/row,tlb_frame_attr=9,title='CONTROL PANEL',xsize=paramx,ysize=240,$
                            xoffset=offset(0)+State.xTop-paramx,yoffset=offset(1)+State.yTop-240+20)
         wParamBase1=widget_base(wParamBase,/column,/frame,xsize=100)
         wParamBase2=widget_base(wParamBase,/column,/frame,xsize=100)
         wParamBase3=widget_base(wParamBase,/column,/frame,xsize=145)
         wParamBase4=widget_base(wParamBase,/column,/frame,xsize=100)
         if (State.ActualObject eq 'cone') then begin
             wParamSlider1=widget_slider(wParamBase1,title='Vrot',value=State.ParamRot,max=10,min=-10,uvalue='VEL1')
             wParamSlider2=widget_slider(wParamBase1,title='Vp',value=State.ParamTex,max=20,min=0,uvalue='VEL2')
             wParamSlider3=widget_slider(wParamBase2,title='RADIUS',value=State.Radius*100,max=100,min=10,uvalue='PAR1')
             wParamSlider4=widget_slider(wParamBase2,title='HEIGHT',value=State.Height*100,max=100,min=10,uvalue='PAR2')
         endif
         if (State.ActualObject eq 'ellipsoid') then begin
             wParamSlider1=widget_slider(wParamBase1,title='Vrot',value=State.ParamRot,max=10,min=-10,uvalue='VEL1')
             wParamSlider2=widget_slider(wParamBase1,title='Vp',value=State.ParamTex,max=20,min=0,uvalue='VEL2')
             wParamSlider3=widget_slider(wParamBase2,title='EXCENTRICITY',value=State.Exc*100,max=90,min=0,uvalue='PAR3')
             wParamSlider4=widget_slider(wParamBase2,title='A',value=State.A*100,max=100,min=10,uvalue='PAR4')
         endif
         if (State.ActualObject eq 'spiral') then begin
             wParamBase5=widget_base(wParamBase,/column,/frame,xsize=100)
             wParamSlider1=widget_slider(wParamBase1,title='V',value=State.V,max=20,min=0,uvalue='VEL3')
             wParamSlider2=widget_slider(wParamBase2,title='HEIGHT',value=State.Height*100,max=100,min=10,uvalue='PAR2')
             wParamSlider3=widget_slider(wParamBase2,title='WIDTH',value=State.Width*100,max=10,min=1,uvalue='PAR5')
             wParamSlider4=widget_slider(wParamBase2,title='R',value=State.R*100,max=50,min=0,uvalue='PAR6')
             wParamSlider5=widget_slider(wParamBase2,title='D',value=100*State.dist,max=100,min=10,uvalue='PAR8')
             wParamSlider6=widget_slider(wParamBase5,title='LEVELS',value=State.Levels,max=1000,min=10,uvalue='NUM1')
             wParamSlider7=widget_slider(wParamBase5,title='ACCURACY',value=State.Accuracy,max=-10,min=-1,uvalue='NUM2')
             wSpace=widget_base(wParamBase5,ysize=100)
             wGenerateButton=widget_button(wParamBase5,value='GENERATE',uvalue='GEN')
         endif
         if (State.ActualObject eq 'cylinder') then begin
             wParamSlider1=widget_slider(wParamBase1,title='Vrot',value=State.ParamRot,max=10,min=-10,uvalue='VEL1')
             wParamSlider2=widget_slider(wParamBase1,title='Vp',value=State.ParamTex,max=20,min=0,uvalue='VEL2')
             wParamSlider3=widget_slider(wParamBase2,title='RADIUS',value=State.Radius*100,max=100,min=10,uvalue='PAR1')
             wParamSlider4=widget_slider(wParamBase2,title='HEIGHT',value=State.Height*100,max=100,min=10,uvalue='PAR2')
         endif

         wParamSlider8=widget_slider(wParamBase3,title='SLIT POSITION',value=100*State.SlitPos,max=100,min=-100,uvalue='SLIT1',/drag)
         wParamSlider9=widget_slider(wParamBase3,title='SLIT WIDTH',value=100*State.SlitWid,max=100,min=1,uvalue='SLIT2')
         wParamSlider10=widget_slider(wParamBase3,title='STEP NUMBER',value=State.Nstep,min=10,max=100,uvalue='QUA3')
         wParamSlider11=widget_slider(wParamBase3,title='LINE WIDTH',value=State.LineWidth,min=30,max=500,uvalue='LINE')
         wToggleBase1=widget_base(wParamBase3,/exclusive)
         wToggleButton1=widget_button(wToggleBase1,value='SHOW CENTRAL LINE',uvalue='BUT1',/no_release)

         wParamSlider12=widget_slider(wParamBase4,title='QUALITY',value=State.density-3,max=15,min=0,uvalue='QUA1')
         wParamSlider13=widget_slider(wParamBase4,title='STEP SIZE',value=State.step*100,max=10,min=1,uvalue='QUA2')
         wParamSlider14=widget_slider(wParamBase4,title='DISTRIBUTION',value=State.Distribution*100,min=0,max=100,uvalue='DIST')
         wSpace=widget_base(wParamBase4,ysize=50)

     ;default
     if (State.ActualObject ne 'spiral') then begin
         wGenerateButton=widget_button(wParamBase4,value='GENERATE',uvalue='GEN')
         widget_control,wParamSlider13,sensitive=0
         wSpace1=widget_base(wParamBase1,ysize=90)
     endif else begin
         widget_control,wParamSlider14,set_value=100
         widget_control,wParamSlider14,sensitive=0
         wSpace1=widget_base(wParamBase1,ysize=140)
     endelse
     if (State.mode eq 1) then widget_control,wGenerateButton,sensitive=0
     widget_control,wParamBase,/realize
     State.wGenerateButton=wGenerateButton
     widget_control,State.wFileButton2,sensitive=1
     widget_control,State.wOptionButton3,sensitive=1
     widget_control,State.wOptionButton2,sensitive=1
     widget_control,State.wOptionButton1,sensitive=1
     State.wPosLabel=widget_label(wParamBase1,value='Position',xsize=90)
     State.wAnglesLabel=widget_label(wParamBase1,value=Get_angles(State.RotAxis),xsize=90)
     State.wParamBase=wParamBase
     State.wToggleButton1=wToggleButton1
     State.wParamSlider=wParamSlider13
     State.ParamExist=1
     widget_control,wParamBase,set_uvalue=State
     widget_control,State.wTopBase,set_uvalue=State

     xmanager,'Param',wParamBase

     ;-------------------Animation-------------------
     pseudoEvent={ID:State.wLeftBase,TOP:State.wTopBase,HANDLER:State.wTopBase}
     Animate_event,pseudoEvent
     end

'CANCEL':begin
         widget_control,event.top,/destroy
         end

endcase
if (widget_info(event.top,/valid_id) eq 1) then widget_control,event.top,set_uvalue=State
widget_control,State.wTopBase,set_uvalue=State
end

;------------------------------- SAVE EVENTS -------------------------------
pro Save_event,event
widget_control,event.id,get_uvalue=uvalue
case uvalue of
'STOP':begin
       widget_control,event.top,get_uvalue=State
       State.SaveAnim=0
       if State.SaveType eq 'MPG' then begin
          widget_control,State.wSaveLabel,set_value='          WAIT! SAVING...'
          mpeg_save,State.oMpeg,filename=State.MainPath+'save\movie.mpg'
          mpeg_close,State.oMpeg
       endif
       widget_control,event.top,/destroy
       widget_control,State.wTopBase,set_uvalue=State
       end
endcase
end

;----------------------------- PROFILE EVENTS -------------------------------
pro Profile_event,event
widget_control,event.id,get_uvalue=uvalue
case uvalue of
'SAVE':begin
       widget_control,event.top,get_uvalue=State
       openw,unit,State.MainPath+'SAVE\profile.dat',/get_lun
       for i=0,n_elements(State.Profile)-1 do printf,unit,State.Profile(i)
       free_lun,unit
       widget_control,event.top,/destroy
       end
'CANCEL':widget_control,event.top,/destroy
endcase
end

;----------------------------- EXAMPLE EVENTS -------------------------------
pro Example_event,event
widget_control,event.id,get_uvalue=uvalue
case uvalue of
'EXAMPLE':begin
       Animate_event,{ID:event.id,TOP:event.top,HANDLER:event.top}
       end
endcase
end

;----------------------------- ANIMATE EVENTS -------------------------------
pro Animate_event,event
widget_control,event.id,get_uvalue=uvalue

case uvalue of
'EXAMPLE':begin
          widget_control,event.top,get_uvalue=State
          widget_control,event.id,get_value=ID_value

          if (ID_value ne '<----') and (ID_value ne '---->') then begin
             State.File=strmid(ID_value,strpos(ID_value,' ')+1,strlen(ID_value)-(strpos(ID_value,' ')+1))
             e_object=strmid(ID_value,0,strpos(ID_value,' '))
             State.ExampleObject=e_object
             State.ExamOffset=[0.0,0.0]
          endif
          if (ID_value eq '<----') then State.File=State.File-1
          if (ID_value eq '---->') then State.File=State.File+1

          e_object=State.ExampleObject
          case e_object of
           'cone':nfile=State.Nfiles(0)
           'ellipsoid':nfile=State.Nfiles(1)
           'spiral':nfile=State.Nfiles(2)
           'cylinder':nfile=State.Nfiles(3)
          endcase

          ;read data
          e_path=State.MainPath+'EXAMPLES\'+e_object+'\example'+strcompress(string(State.File),/remove_all)
          openr,unit,e_path+'.dat',/get_lun
          ActualObject=' ' & Matrix=fltarr(4,4) & RotAxis=fltarr(3) & ParamTex=0L & ParamRot=0L & V=0
          Radius=0.0 & Height=0.0 & R=0.0 & Exc=0.0 & A=0.0 & Width=0.0 & Dist=0.0
          SlitPos=0.0 & SlitWid=0.0 & Levels=0 & Accuracy=0 & Distribution=0.0
          Step=0.0 & Nstep=0 & Density=0 & ActualTexture=' ' & LineWidth=0 & Line=' '
          readf,unit,ActualObject
          readf,unit,Matrix
          readf,unit,RotAxis
          readf,unit,ParamTex,ParamRot,V,Radius,Height,R,Exc,A,Width,Dist,SlitPos,SlitWid,Levels,Accuracy,$
                Distribution,Step,Nstep,Density,LineWidth
          readf,unit,Line
          State.ActualObject=ActualObject & State.Matrix=Matrix & State.ParamTex=ParamTex & State.ParamRot=ParamRot
          State.V=V & State.Radius=Radius & State.Height=Height & State.R=R &  State.Exc=Exc & State.A=A
          State.Width=Width & State.Dist=Dist & State.SlitPos=SlitPos & State.SlitWid=SlitWid & State.Levels=Levels
          State.Accuracy=Accuracy & State.Distribution=Distribution & State.Step=Step & State.Nstep=Nstep
          State.Density=Density & State.RotAxis=RotAxis
          free_lun,unit
          read_jpeg,e_path+'.jpg',ExampleSpectra

          ;------------------position-----------------
          if (ID_value ne '<----') and (ID_value ne '---->') then begin
           if widget_info(State.wParamBase,/valid) then begin
             widget_control,State.wParamBase,tlb_get_offset=ParamOffset
             State.ParamOffset=ParamOffset
             widget_control,State.wParamBase,/destroy
           endif
          endif

          ;------------------widgets------------------
          if (ID_value ne '<----') and (ID_value ne '---->') then begin
           widget_control,State.wInnerBase,/destroy
           State.wInnerBase=widget_base(State.wTopBase,/row)
              State.wLeftBase=widget_base(State.wInnerBase,uvalue='TIMER')
                  State.wDraw=widget_draw(State.wLeftBase,xsize=State.xdim,ysize=State.ydim,$
                                        graphics_level=2,retain=0,/button_events,$
                                        /expose_events,uvalue='DRAW')
              State.wRightBase=widget_base(State.wInnerBase)
                  State.wRightDraw=widget_draw(State.wRightBase,xsize=State.xdim,ysize=State.ydim,$
                                               /button_events,/motion_events,uvalue='RIGHT')
          endif

          widget_control,State.wRightDraw,get_value=Window1
          wset,Window1
          State.RightDraw=Window1
          tvscl,congrid(ExampleSpectra,State.xdim,State.ydim)

          if (widget_info(State.wExampleBase,/valid_id)) then begin
             widget_control,State.wExampleBase,tlb_get_offset=ExamOffset
             State.ExamOffset=ExamOffset
             widget_control,State.wExampleBase,/destroy
          endif

          if (total(State.ExamOffset) eq 0.0) then begin
             widget_control,State.wTopBase,tlb_get_offset=offset
             offset(1)=offset(1)+State.ydim
          endif else offset=State.ExamOffset

          State.wExampleBase=widget_base(/row,tlb_frame_attr=1+8+2,xoffset=offset(0),$
                                         yoffset=offset(1),title='EXAMPLES') ;1 2 4 8
                wPrevButton=widget_button(State.wExampleBase,value='<----',uvalue='EXAMPLE',xsize=50)
                wNextButton=widget_button(State.wExampleBase,value='---->',uvalue='EXAMPLE',xsize=50)
          widget_control,State.wExampleBase,/realize
          if (State.File eq 1) and (widget_info(wPrevButton,/valid_id)) then widget_control,wPrevButton,sensitive=0
          if (State.File eq nfile) and (widget_info(wNextButton,/valid_id)) then widget_control,wNextButton,sensitive=0
          xmanager,'Example',State.wExampleBase

          ;-----------------objects-------------------
          if (ID_value ne '<----') and (ID_value ne '---->') then begin
             obj_destroy,State.oHolder
             obj_destroy,State.oWindow
          endif else obj_destroy,State.oHolder

          ;oImage
          read_jpeg,State.ActualTexture,image
          oImage=obj_new('IDLgrImage',image)
          oImagespiral=obj_new('IDLgrImage',image(*,0:50,*))

          ;oObject
          if (State.ActualObject eq 'cone') then begin
                                     oObject=obj_new('cone',color=[150,150,150],radius=State.Radius,height=State.Height,$
                                         texture_map=oImage,bottom=[215,215,215],density=State.density)
                                     oBottom=obj_new('cone_bottom',color=[255,255,255],radius=State.Radius,$
                                         height=State.Height,texture_map=oImage,bottom=[255,255,255],density=State.density,$
                                         inner=State.Distribution)
                                     oInner=obj_new('cone',color=[150,150,150],radius=State.Radius-State.Radius*State.Distribution, $
                                         height=State.Height,texture_map=oImage,bottom=[215,215,215],density=State.density)
                                     endif
          if (State.ActualObject eq 'ellipsoid') then begin
                                     oObject=obj_new('ellipsoid',color=[150,150,150],texture_map=oImage,$
                                         bottom=[215,215,215],density=State.density,exc=State.Exc,a=state.A)
                                     oBottom=obj_new('ellipsoid_bottom',color=[255,255,255],texture_map=oImage,$
                                         bottom=[255,255,255],density=State.density,exc=State.Exc,a=state.A,$
                                         inner=State.Distribution)
                                     b=State.A*sqrt(1-(State.Exc)^2)
                                     exc2=sqrt(1-(b-b*State.Distribution)^2/(State.A)^2)
                                     oInner=obj_new('ellipsoid',color=[150,150,150],texture_map=oImage,$
                                         bottom=[215,215,215],density=State.density,exc=exc2,a=state.A)
                                     endif
          if (State.Width eq 0.0) then spiralstyle=1 else spiralstyle=2
          if (State.ActualObject eq 'spiral') then begin
                                     oObject=obj_new('spiral',color=[150,150,150],bottom=[215,215,215],$
                                         texture_map=oImagespiral,density=State.density,height=State.Height,width=State.Width,$
                                         r=State.R,dist=State.Dist,style=spiralstyle)
                                     oBottom=obj_new('spiral_bottom',color=[255,255,255],bottom=[255,255,255],$
                                         texture_map=oImagespiral,density=State.density,height=State.Height,width=State.Width,$
                                         r=State.R,dist=State.Dist,style=spiralstyle)
                                     oBottom2=obj_new('spiral_bottom',color=[255,255,255],bottom=[255,255,255],$
                                         texture_map=oImagespiral,density=State.density,height=State.Height,width=State.Width,$
                                         r=State.R,dist=State.Dist,style=spiralstyle,pos=[0.0,0.0,State.Height])
                                     endif
          if (State.ActualObject eq 'cylinder') then begin
                                     oObject=obj_new('cylinder',color=[150,150,150],bottom=[215,215,215],$
                                         texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius)
                                         oBottom=obj_new('cylinder_bottom',color=[255,255,255],bottom=[255,255,255],$
                                         texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius,$
                                         inner=State.Distribution)
                                     oBottom2=obj_new('cylinder_bottom',color=[255,255,255],bottom=[255,255,255],$
                                         texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius,$
                                         pos=[0.0,0.0,State.Height],inner=State.Distribution)
                                     oInner=obj_new('cylinder',color=[150,150,150],bottom=[215,215,215],$
                                         texture_map=oImage,density=State.density,height=State.Height,radius=State.Radius-$
                                         State.Radius*State.Distribution)
                                     endif

          ;oSlit
          oSlit1=obj_new('line',pos=State.SlitPos)
          oSlit2=obj_new('line',pos=State.SlitPos+State.SlitWid-0.01)

          ;texture
          oObject->SetProperty,texture_coord=Make_coord(State)
          oBottom->SetProperty,texture_coord=Make_bottom_coord(State)

          if (State.ActualObject ne 'spiral') then begin
             if State.Distribution eq 0.0 then oBottom->SetProperty,hide=1
             oInner->SetProperty,texture_coord=Make_coord(State)
             if State.Distribution eq 0.0 then oInner->SetProperty,hide=1
          endif
          if (State.ActualObject eq 'cylinder') then begin
             oBottom2->SetProperty,texture_coord=Make_bottom_coord(State)
             if State.Distribution eq 0.0 then oBottom2->SetProperty,hide=1
          endif

          ;lights
          oLight1=obj_new('IDLgrLight',type=0,intensity=1)
          oLight=obj_new('IDLgrModel')
          oLight->Add,oLight1

          ;oModel
          oModel=obj_new('IDLgrModel')
          oModel->Add,oObject
          oModel->Add,oBottom
          if (State.ActualObject eq 'cylinder') or (State.ActualObject eq 'spiral') then oModel->Add,oBottom2
          if (State.ActualObject ne 'spiral') then oModel->Add,oInner
          oModel->Add,oLight
          oModel->SetProperty,transform=State.Matrix

          ;oView
          oView=obj_new('IDLgrView',color=[0,0,0],zclip=[2,-2])
          oView->Add,oModel
          oView->Add,oSlit1
          oView->Add,oSlit2

          ;oWindow
          widget_control,State.wDraw,get_value=oWindow
          oWindow->Draw,oView

          ;container
          oHolder=obj_new('IDLgrContainer')
          oHolder->Add,oImage
          oHolder->Add,oImagespiral
          oHolder->Add,oView

          ;State
          State.oImage=oImage
          State.oImagespiral=oImagespiral
          State.oObject=oObject
          State.oBottom=oBottom
          State.oModel=oModel
          State.oLight=oLight
          State.oView=oView
          State.oHolder=oHolder
          State.oWindow=oWindow
          State.oSlit1=oSlit1
          State.oSlit2=oSlit2
          State.RunAnim=0
          if (State.ActualObject ne 'spiral') then State.oInner=oInner
          if (State.ActualObject eq 'cylinder') or (State.ActualObject eq 'spiral') then State.oBottom2=oBottom2

          ;---------------Parameters Base------------------------
          if (total(State.ParamOffset) eq 0.0) then begin
             widget_control,State.wTopBase,tlb_get_offset=offset
             if (State.ActualObject eq 'spiral') then paramx=570 else paramx=465
             offset(0)=offset(0)+State.xTop-paramx
             offset(1)=offset(1)+State.yTop-240+20
          endif else offset=State.ParamOffset

         if (ID_value ne '<----') and (ID_value ne '---->') then begin
         wParamBase=widget_base(/row,tlb_frame_attr=9,title='CONTROL PANEL',xsize=paramx,ysize=240,$
                                xoffset=offset(0),yoffset=offset(1))
           wParamBase1=widget_base(wParamBase,/column,/frame,xsize=100)
           wParamBase2=widget_base(wParamBase,/column,/frame,xsize=100)
           wParamBase3=widget_base(wParamBase,/column,/frame,xsize=145)
           wParamBase4=widget_base(wParamBase,/column,/frame,xsize=100)
           if (State.ActualObject eq 'cone') then begin
               wParamSlider1=widget_slider(wParamBase1,title='Vrot',value=State.ParamRot,max=10,min=-10,uvalue='VEL1')
               wParamSlider2=widget_slider(wParamBase1,title='Vp',value=State.ParamTex,max=20,min=0,uvalue='VEL2')
               wParamSlider3=widget_slider(wParamBase2,title='RADIUS',value=State.Radius*100,max=100,min=10,uvalue='PAR1')
               wParamSlider4=widget_slider(wParamBase2,title='HEIGHT',value=State.Height*100,max=100,min=10,uvalue='PAR2')
           endif
           if (State.ActualObject eq 'ellipsoid') then begin
               wParamSlider1=widget_slider(wParamBase1,title='Vrot',value=State.ParamRot,max=10,min=-10,uvalue='VEL1')
               wParamSlider2=widget_slider(wParamBase1,title='Vp',value=State.ParamTex,max=20,min=0,uvalue='VEL2')
               wParamSlider3=widget_slider(wParamBase2,title='EXCENTRICITY',value=State.Exc*100,max=90,min=0,uvalue='PAR3')
               wParamSlider4=widget_slider(wParamBase2,title='A',value=State.A*100,max=100,min=10,uvalue='PAR4')
           endif
           if (State.ActualObject eq 'spiral') then begin
               wParamBase5=widget_base(wParamBase,/column,/frame,xsize=100)
               wParamSlider1=widget_slider(wParamBase1,title='V',value=State.V,max=20,min=0,uvalue='VEL3')
               wParamSlider2=widget_slider(wParamBase2,title='HEIGHT',value=State.Height*100,max=100,min=10,uvalue='PAR2')
               wParamSlider3=widget_slider(wParamBase2,title='WIDTH',value=State.Width*100,max=10,min=1,uvalue='PAR5')
               wParamSlider4=widget_slider(wParamBase2,title='R',value=State.R*100,max=50,min=0,uvalue='PAR6')
               wParamSlider5=widget_slider(wParamBase2,title='D',value=100*State.dist,max=100,min=10,uvalue='PAR8')
               wParamSlider6=widget_slider(wParamBase5,title='LEVELS',value=State.Levels,max=1000,min=10,uvalue='NUM1')
               wParamSlider7=widget_slider(wParamBase5,title='ACCURACY',value=State.Accuracy,max=-10,min=-1,uvalue='NUM2')
               wSpace=widget_base(wParamBase5,ysize=100)
               State.wParamSlider5=wParamSlider5
               State.wParamSlider6=wParamSlider6
               State.wParamSlider7=wParamSlider7
           endif
           if (State.ActualObject eq 'cylinder') then begin
               wParamSlider1=widget_slider(wParamBase1,title='Vrot',value=State.ParamRot,max=10,min=-10,uvalue='VEL1')
               wParamSlider2=widget_slider(wParamBase1,title='Vp',value=State.ParamTex,max=20,min=0,uvalue='VEL2')
               wParamSlider3=widget_slider(wParamBase2,title='RADIUS',value=State.Radius*100,max=100,min=10,uvalue='PAR1')
               wParamSlider4=widget_slider(wParamBase2,title='HEIGHT',value=State.Height*100,max=100,min=10,uvalue='PAR2')
           endif

           wParamSlider8=widget_slider(wParamBase3,title='SLIT POSITION',value=100*State.SlitPos,max=100,min=-100,uvalue='SLIT1',/drag)
           wParamSlider9=widget_slider(wParamBase3,title='SLIT WIDTH',value=100*State.SlitWid,max=100,min=1,uvalue='SLIT2')
           wParamSlider10=widget_slider(wParamBase3,title='STEP NUMBER',value=State.Nstep,min=10,max=100,uvalue='QUA3')
           wParamSlider11=widget_slider(wParamBase3,title='LINE WIDTH',value=State.LineWidth,min=30,max=500,uvalue='LINE')
           wToggleBase1=widget_base(wParamBase3,/exclusive)
           wToggleButton1=widget_button(wToggleBase1,value='SHOW CENTRAL LINE',uvalue='BUT1',/no_release)
           wParamSlider12=widget_slider(wParamBase4,title='QUALITY',value=State.density-3,max=15,min=0,uvalue='QUA1')
           wParamSlider13=widget_slider(wParamBase4,title='STEP SIZE',value=State.step*100,max=10,min=1,uvalue='QUA2')
           wParamSlider14=widget_slider(wParamBase4,title='DISTRIBUTION',value=State.Distribution*100,min=0,max=100,uvalue='DIST')
           wSpace=widget_base(wParamBase4,ysize=50)
           widget_control,wParamBase,/realize

           State.wToggleButton1=wToggleButton1
           State.wParamSlider1=wParamSlider1
           State.wParamSlider2=wParamSlider2
           State.wParamSlider3=wParamSlider3
           State.wParamSlider4=wParamSlider4
           State.wParamSlider8=wParamSlider8
           State.wParamSlider9=wParamSlider9
           State.wParamSlider10=wParamSlider10
           State.wParamSlider11=wParamSlider11
           State.wParamSlider12=wParamSlider12
           State.wParamSlider13=wParamSlider13
           State.wParamSlider14=wParamSlider14

           ;default
           if (State.ActualObject ne 'spiral') then begin
             wSpace1=widget_base(wParamBase1,ysize=105)
             widget_control,State.wParamSlider13,sensitive=0
           endif else begin
             widget_control,State.wParamSlider14,set_value=100
             widget_control,State.wParamSlider14,sensitive=0
             wSpace1=widget_base(wParamBase1,ysize=155)
           endelse
           widget_control,wParamBase3,sensitive=0
           widget_control,wParamBase4,sensitive=0
           widget_control,wParamSlider1,sensitive=0
           widget_control,wParamSlider2,sensitive=0
           widget_control,wParamBase2,sensitive=0
           if State.ActualObject eq 'spiral' then widget_control,wParamBase5,sensitive=0
           State.wAnglesLabel=widget_label(wParamBase1,value=Get_angles(State.RotAxis),xsize=90)
           State.wParamBase=wParamBase
           State.wParamSlider=wParamSlider13
           State.ParamExist=1

         endif else begin
           if (State.ActualObject eq 'cone') or (State.ActualObject eq 'cylinder') then begin
              widget_control,State.wParamSlider1,set_value=State.ParamRot
              widget_control,State.wParamSlider2,set_value=State.ParamTex
              widget_control,State.wParamSlider3,set_value=State.Radius*100
              widget_control,State.wParamSlider4,set_value=State.Height*100
           endif
           if (State.ActualObject eq 'ellipsoid') then begin
              widget_control,State.wParamSlider1,set_value=State.ParamRot
              widget_control,State.wParamSlider2,set_value=State.ParamTex
              widget_control,State.wParamSlider3,set_value=State.Exc*100
              widget_control,State.wParamSlider4,set_value=State.A*100
           endif
           if (State.ActualObject eq 'spiral') then begin
              widget_control,State.wParamSlider1,set_value=State.V
              widget_control,State.wParamSlider2,set_value=State.Height*100
              widget_control,State.wParamSlider3,set_value=State.Width*100
              widget_control,State.wParamSlider4,set_value=State.R*100
              widget_control,State.wParamSlider5,set_value=State.dist*100
              widget_control,State.wParamSlider6,set_value=State.Levels
              widget_control,State.wParamSlider7,set_value=State.Accuracy
           endif
           widget_control,State.wParamSlider8,set_value=State.SlitPos*100
           widget_control,State.wParamSlider9,set_value=State.SlitWid*100
           widget_control,State.wParamSlider10,set_value=State.Nstep
           widget_control,State.wParamSlider11,set_value=State.LineWidth
           widget_control,State.wParamSlider12,set_value=State.density-3
           widget_control,State.wParamSlider13,set_value=State.step*100
           widget_control,State.wParamSlider14,set_value=State.Distribution*100
         endelse

         widget_control,State.wDraw,sensitive=0
         widget_control,State.wRightDraw,sensitive=0
         widget_control,State.wOptionButton3,sensitive=0
         widget_control,State.wOptionButton2,sensitive=0
         widget_control,State.wOptionButton1,sensitive=0
         widget_control,State.wFileButton2,sensitive=0
         if (Line eq 'ERASE CENTRAL LINE') then widget_control,State.wToggleButton1,/set_button
         widget_control,State.wAnglesLabel,set_value=Get_angles(State.RotAxis)
         widget_control,State.wParamBase,set_uvalue=State
         widget_control,State.wTopBase,set_uvalue=State
         widget_control,State.wExampleBase,set_uvalue=State
         xmanager,'Param',State.wParamBase

         ;-------------------Animation-------------------
         pseudoEvent={ID:State.wLeftBase,TOP:State.wTopBase,HANDLER:State.wTopBase}
         Animate_event,pseudoEvent
         end

'OBJECT':begin
          widget_control,event.top,get_uvalue=State
          widget_control,State.wTopBase,sensitive=0
          if widget_info(State.wParamBase,/valid_id) then widget_control,State.wParamBase,sensitive=0

          ;widgets
          dimObject=State.dimObject
          widget_control,State.wTopBase,tlb_get_offset=offset
          wObjectBase=widget_base(/column,xoffset=offset(0),yoffset=offset(1),title='OBJECT')
              wObjectRow1=widget_base(wObjectBase,/row)
                  wObjectDraw1=widget_draw(wObjectRow1,xsize=dimObject,ysize=dimObject,/button_events,uvalue='DRAW1')
                  wObjectDraw2=widget_draw(wObjectRow1,xsize=dimObject,ysize=dimObject,/button_events,uvalue='DRAW2')
                  wObjectDraw3=widget_draw(wObjectRow1,xsize=dimObject,ysize=dimObject,/button_events,uvalue='DRAW3')
                  wObjectDraw4=widget_draw(wObjectRow1,xsize=dimObject,ysize=dimObject,/button_events,uvalue='DRAW4')
              wObjectRow2=widget_base(wObjectBase,/row)
                  wObjectLabel=widget_label(wObjectRow2,value=State.ActualObject,/align_left,xsize=2*dimObject)
              wObjectRow3=widget_base(wObjectBase,/row)
                  wObjectButton1=widget_button(wObjectRow3,value='OK',uvalue='OK')
                  wObjectButton2=widget_button(wObjectRow3,value='CANCEL',uvalue='CANCEL')
          widget_control,wObjectBase,/realize

          ;load images
          read_jpeg,State.ObjectPath(0),image1     ;1st Object
          widget_control,wObjectDraw1,get_value=window1 & wset,window1
          tv,congrid(image1,3,dimObject,dimObject),true=1
          read_jpeg,State.ObjectPath(1),image2    ;2nd Object
          widget_control,wObjectDraw2,get_value=window2 & wset,window2
          tv,congrid(image2,3,dimObject,dimObject),true=1
          read_jpeg,State.ObjectPath(2),image3    ;3rd Object
          widget_control,wObjectDraw3,get_value=window3 & wset,window3
          tv,congrid(image3,3,dimObject,dimObject),true=1
          read_jpeg,State.ObjectPath(3),image4    ;4th Object
          widget_control,wObjectDraw4,get_value=window4 & wset,window4
          tv,congrid(image4,3,dimObject,dimObject),true=1

          State.wObjectLabel=wObjectLabel
          widget_control,wObjectBase,set_uvalue=State
          widget_control,State.wTopBase,set_uvalue=State
          xmanager,'Object',wObjectBase,cleanup='Object_cleanup'
          end

'TEXTURE':begin
          widget_control,event.top,get_uvalue=State
          widget_control,State.wTopBase,sensitive=0
          if widget_info(State.wParamBase,/valid_id) then widget_control,State.wParamBase,sensitive=0

          ;widgets
          dimTexture=State.dimTexture
          widget_control,State.wTopBase,tlb_get_offset=offset
          wTextureBase=widget_base(/column,xoffset=offset(0),yoffset=offset(1),title='TEXTURE')
              wTextureRow1=widget_base(wTextureBase,/row)
                  wTextureDraw1=widget_draw(wTextureRow1,xsize=dimTexture,ysize=dimTexture,/button_events,uvalue='DRAW1')
                  wTextureDraw2=widget_draw(wTextureRow1,xsize=dimTexture,ysize=dimTexture,/button_events,uvalue='DRAW2')
                  wTextureDraw3=widget_draw(wTextureRow1,xsize=dimTexture,ysize=dimTexture,/button_events,uvalue='DRAW3')
              wTextureRow2=widget_base(wTextureBase,/row)
                  wTextureLabel=widget_label(wTextureRow2,value=State.ActualTexture,/align_left,xsize=3*dimTexture)
              wTextureRow3=widget_base(wTextureBase,/row)
                  wTextureButton1=widget_button(wTextureRow3,value='OK',uvalue='OK')
                  wTextureButton2=widget_button(wTextureRow3,value='CANCEL',uvalue='CANCEL')
          widget_control,wTextureBase,/realize

          ;load images
          read_jpeg,State.TexturePath(0),image1    ;1st texture
          widget_control,wTextureDraw1,get_value=window1 & wset,window1
          tv,congrid(image1,3,dimTexture,dimTexture),true=1
          read_jpeg,State.TexturePath(1),image2    ;2nd texture
          widget_control,wTextureDraw2,get_value=window2 & wset,window2
          tv,congrid(image2,3,dimTexture,dimTexture),true=1
          read_jpeg,State.TexturePath(2),image3    ;3rd texture
          widget_control,wTextureDraw3,get_value=window3 & wset,window3
          tv,congrid(image3,3,dimTexture,dimTexture),true=1

          State.wTextureLabel=wTextureLabel
          widget_control,wTextureBase,set_uvalue=State
          widget_control,State.wTopBase,set_uvalue=State
          xmanager,'Texture',wTextureBase,cleanup='Texture_cleanup'
          end

'TIMER':begin
            widget_control,Event.top,get_uvalue=State
            ;-------------------texture motion-----------------------
            case State.ActualObject of
                 'cone':shifttex=State.ParamTex*0.001/(2*sqrt(State.Height^2+State.Radius^2))
                 'cylinder':shifttex=State.ParamTex*0.001/(2*State.Height)
                 'ellipsoid':shifttex=State.ParamTex*0.001/(0.5*(2*!pi*State.A*(1-(State.Exc/2)^2-(3.0/8.0)^2*State.Exc^4/3-(15.0/48.0)^2*State.Exc^6/5)))
                 'spiral':shifttex=State.V*0.001/(2*(State.Height*sqrt((2*!pi*State.R/State.dist)^2+1)))
            endcase
            State.up=State.up-shifttex
            State.down=State.down-shifttex
            if (State.down lt 0.0) then begin
                State.up=1.0
                State.down=0.5
            endif
            State.oObject->SetProperty,texture_coord=Make_coord(State)
            if (State.ActualObject ne 'spiral') then State.oInner->SetProperty,texture_coord=Make_coord(State)

            if (State.ActualObject eq 'cone') and (State.Distribution ne 0.0) then begin ;bottom texture motion
               shifttex=State.ParamTex*0.001*State.Radius/(2*(State.Height^2+State.Radius^2))
               State.up_bottom=State.up_bottom-shifttex
               State.down_bottom=State.down_bottom-shifttex
               if (State.down_bottom lt 0.0) then begin
                  State.up_bottom=1.0
                  State.down_bottom=0.5
               endif
              State.oBottom->SetProperty,texture_coord=Make_bottom_coord(State)
            endif

            ;------------------rotational motion----------------------
            if (State.ActualObject ne 'spiral') then State.oModel->Rotate,State.RotAxis,State.ParamRot ;oObject

            State.oWindow->Draw,State.oView
            widget_control,State.wLeftBase,timer=State.Timer

            ;----------------saving animation-----------------------------
            if State.SaveAnim ne 0 then begin
               SaveIm=State.oWindow->read()
               SaveIm->GetProperty,data=data
               if State.SaveType eq 'MPG' then mpeg_put,State.oMpeg,image=data,frame=State.SaveAnim,/order $
                  else write_jpeg,State.MainPath+'save\'+strcompress(string(State.SaveAnim),/remove_all)+'.jpg',data,true=1 ;(State.saveanim+100)
               State.SaveAnim=State.SaveAnim+1
               widget_control,State.wSaveLabel,set_value='          FRAME NUMBER'+strcompress(string(State.SaveAnim),/remove_all)
                ;if State.SaveAnim eq 300 then begin
                ;   widget_control,State.wSaveLabel,set_value='       ENDING ANIMATION'+strcompress(string(State.SaveAnim),/remove_all)
                ;   wait,2
                ;   Save_event,{ID:State.wSaveButton,TOP:State.wSaveBase,HANDLER:State.wSaveBase}
                ;endif
            endif
            widget_control,Event.top,set_uvalue=State
        end

'DRAW':begin
       widget_control,event.top,get_uvalue=State

       if (State.RunAnim eq 1) then begin

        ;trackball update
        bHaveXform = State.oTrack->Update(Event, TRANSFORM=TrackXform)
        IF (bHaveXform) THEN BEGIN
        State.oModel->GetProperty, TRANSFORM=ModelXform
        State.oModel->SetProperty, TRANSFORM=ModelXform # TrackXform
        State.Matrix=State.Matrix # TrackXform
        State.RotAxis=State.RotAxis # TrackXform(0:2,0:2)
        State.oWindow->Draw, State.oView
        ENDIF

        ;button press
        if (Event.type eq 0) then begin
           State.btndown=1b
           State.oWindow->SetProperty,quality=0
           widget_control,State.wDraw,/draw_motion
        endif

        ;button motion
        if (Event.type eq 2) then begin
           if (bHaveXform) then State.oWindow->Draw,State.oView
        endif

        ;button release
        if (Event.type eq 1) then begin
           if (State.btndown eq 1b) then begin
           State.oWindow->SetProperty,quality=2
           State.oWindow->Draw,State.oView
           endif
           State.btndown=0b
           widget_control,State.wDraw,draw_motion=0
           if State.mode eq 1 then Generate,State else redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
           widget_control,State.wAnglesLabel,set_value=Get_angles(State.RotAxis)
        endif

        widget_control,Event.top,set_uvalue=State
      endif
      end

'RIGHT':begin ;prave okno
        widget_control,event.top,get_uvalue=State
        if widget_info(State.wPosLabel,/valid_id) then $ ;pozice kurzoru
           widget_control,State.wPosLabel,set_value='Position '+strcompress('('+string(event.x)+','+$
                                                  string(event.y)+')',/remove_all)
        if (event.press eq 1) then begin                 ;leve tlacitko mysi
           widget_control,State.wTopBase,tlb_get_offset=offset
           profilbase=widget_base(/column,xoffset=offset(0)+State.xTop-State.xdim-7,yoffset=offset(1),$
                                  tlb_frame_attr=1,title='PROFILE')
            profildraw=widget_draw(profilbase,xsize=State.xdim,ysize=State.xdim/2)
            buttonbase=widget_base(profilbase,/row)
             button1=widget_button(buttonbase,value='SAVE',uvalue='SAVE')
             button2=widget_button(buttonbase,value='CANCEL',uvalue='CANCEL')
           widget_control,profilbase,/realize
           widget_control,profildraw,get_value=Window1 & wset,Window1
           profile=State.spectra(*,event.y)
           plot,profile,pos=[0.0,0.0,1.0,1.0],xstyle=4,ystyle=4
           State.Profile=profile
           widget_control,State.wTopBase,set_uvalue=State
           widget_control,profilbase,set_uvalue=State
           xmanager,'Profile',profilbase
        endif
        end

 'ON':begin ;on mode
      widget_control,event.top,get_uvalue=State
      if widget_info(State.wParamBase,/valid_id) then begin
         Generate,State
         widget_control,State.wGenerateButton,sensitive=0
      endif
      State.Mode=1
      widget_control,Event.top,set_uvalue=State
      end

'OFF':begin ;off mode
      widget_control,event.top,get_uvalue=State
      if widget_info(State.wParamBase,/valid_id) then begin
         widget_control,State.wRightDraw,get_value=Window1
         wset,Window1
         redraw,State.xdim,State.ydim,State.wRightDraw,/winerase,condition=State.wToggleButton1
         widget_control,State.wGenerateButton,sensitive=1
      endif
      State.Mode=0
      widget_control,Event.top,set_uvalue=State
      end

'SAVE':begin ;saving animation
       widget_control,event.top,get_uvalue=State
       widget_control,event.id,get_value=Val & State.SaveType=Val(0)
       State.SaveAnim=1
       wset,State.RightDraw
       write_jpeg,State.MainPath+'save\spectra.jpg',tvrd()
       widget_control,State.wTopBase,tlb_get_offset=offset
       wSaveBase=widget_base(xsize=250,ysize=70,tlb_frame_attr=1+8+2+4,/base_align_center,/row,$
                             xoffset=offset(0)+State.xTop*0.65,yoffset=offset(1)+State.yTop*0.5)
         wSaveBase1=widget_base(wSaveBase,xsize=200,ysize=30,/row,/base_align_center)
           SaveLabel=widget_label(wSaveBase1,value='          FRAME NUMBER 1',xsize=150)
           SaveButton=widget_button(wSaveBase1,value='STOP',uvalue='STOP')
       widget_control,wSaveBase,/realize

       if State.SaveType eq 'MPG' then begin
          MpegID=mpeg_open([State.xdim,State.ydim])
          State.oMpeg=MpegID
       endif

       State.wSaveLabel=SaveLabel
       widget_control,State.wParamBase,sensitive=0
       widget_control,State.wTopBase,sensitive=0
       widget_control,Event.top,set_uvalue=State
       widget_control,wSaveBase,set_uvalue=State
       xmanager,'Save',wSaveBase,cleanup='Save_cleanup'
       end

'INVERT':begin ;background color
         widget_control,event.top,get_uvalue=State
         if State.Negativ eq 0 then begin
              State.Negativ=1
              col=[255,255,255]
         endif else begin
              State.Negativ=0
              col=[0,0,0]
         endelse
         State.oView->SetProperty,color=col
         widget_control,State.wRightDraw,get_value=Window1 & wset,Window1
         if(State.Negativ eq 0) then tv,bytscl(State.spectra) else tv,255-bytscl(State.spectra)
         redraw,State.xdim,State.ydim,State.wRightDraw,condition=State.wToggleButton1
         widget_control,event.top,set_uvalue=State
         end

'HELP':begin ;help
       widget_control,event.top,get_uvalue=State
       read_jpeg,State.DataPath+'help.jpg',helpimage
       s=size(helpimage)
       widget_control,State.wTopBase,tlb_get_offset=offset
       wTextBase=widget_base(/column,title='HELP',tlb_frame_attr=1,xoffset=offset(0),yoffset=offset(0)) ;zadny zvetsovani
          wTextDraw=widget_draw(wTextBase,xsize=s(2),ysize=s(3),scr_xsize=s(2)+15,scr_ysize=600,/scroll)
       widget_control,wTextBase,/realize
       widget_control,wTextDraw,get_value=helpwindow & wset,helpwindow
       tv,helpimage,true=1
       State.wHelpBase=wTextBase
       widget_control,event.top,set_uvalue=State
       end

'WRITE':begin ;saving examples
        widget_control,event.top,get_uvalue=State
        widget_control,State.wToggleButton1,get_value=line
        case State.ActualObject of
        'cone':begin
                   nfile=State.Nfiles(0)
                   State.Nfiles(0)=State.Nfiles(0)+1
                   helpbutton=State.wConeButton
                   end
        'ellipsoid':begin
                   nfile=State.Nfiles(1)
                   State.Nfiles(1)=State.Nfiles(1)+1
                   helpbutton=State.wellipsoidButton
                   end
        'spiral':begin
                   nfile=State.Nfiles(2)
                   State.Nfiles(2)=State.Nfiles(2)+1
                   helpbutton=State.wSpiralButton
                   end
        'cylinder':begin
                   nfile=State.Nfiles(3)
                   State.Nfiles(3)=State.Nfiles(3)+1
                   helpbutton=State.wCylinderButton
                   end
        endcase
        e_path=State.MainPath+'EXAMPLES\'+State.ActualObject+'\example'+strcompress(string(nfile+1),/remove_all)
        openw,unit,e_path+'.dat',/get_lun
        printf,unit,State.ActualObject
        printf,unit,State.Matrix
        printf,unit,State.RotAxis
        printf,unit,State.ParamTex,State.ParamRot,State.V,$
               State.Radius,State.Height,State.R,State.Exc,State.A,State.Width,State.Dist,$
               State.SlitPos,State.SlitWid,State.Levels,State.Accuracy,State.Distribution,$
               State.Step,State.Nstep,State.Density,State.LineWidth
        printf,unit,line
        free_lun,unit
        widget_control,State.wRightDraw,get_value=Window1 & wset,Window1
        write_jpeg,e_path+'.jpg',tvrd()
        examplename=State.ActualObject+' '+strcompress(string(nfile+1),/remove_all)
        wExampleButtonNext=widget_button(helpbutton,value=examplename,uvalue='EXAMPLE')
        widget_control,event.top,set_uvalue=State
        end

'QUIT':begin
       widget_control,event.top,/destroy
       end

endcase
end

;----------------------------- ANIMATE ------------------------------
pro Start_program

rinfo=routine_info('Start_program',/source)
mainpath=rinfo.path
mainpath=strmid(mainpath,0,strlen(mainpath)-25)
datapath=mainpath+'data\'
resolve_routine,'Generate'
cone_define
ellipsoid_define
line_define
spiral_define
cylinder_define
cone_bottom_define
ellipsoid_bottom_define
cylinder_bottom_define
spiral_bottom_define

xTop=1000 ;alespon 300
xdim=xTop/2-4
if xdim mod 2 eq 1 then xdim=xdim-1
ydim=xdim
yTop=ydim+9
over=200
!except=0 ;never report exception (floating underflow)

TexturePath=[datapath+'texture1.jpg',datapath+'texture2.jpg',datapath+'texture3.jpg']
ObjectPath=[datapath+'cone.jpg',datapath+'ellipsoid.jpg',datapath+'spiral.jpg',datapath+'cylinder.jpg']
ExampleFiles1=findfile(mainpath+'\examples\cone\*.dat',count=nf1)
ExampleFiles2=findfile(mainpath+'\examples\ellipsoid\*.dat',count=nf2)
ExampleFiles3=findfile(mainpath+'\examples\spiral\*.dat',count=nf3)
ExampleFiles4=findfile(mainpath+'\examples\cylinder\*.dat',count=nf4)

;----------------------------widgets-------------------------
wTopBase=widget_base(mbar=bar,tlb_frame_attr=1,xsize=xTop,ysize=yTop,title='OBJECTS')
    wFileMenu=widget_button(bar,value='FILE',/menu)
        wFileButton1=widget_button(wFileMenu,value='OBJECT TYPE',uvalue='OBJECT')
        wFileButton2=widget_button(wFileMenu,value='SAVE ANIMATION AS',/menu)
            wSaveButton1=widget_button(wFileButton2,value='JPG',uvalue='SAVE')
            wSaveButton2=widget_button(wFileButton2,value='MPG',uvalue='SAVE')
        wFileButton3=widget_button(wFileMenu,value='QUIT',uvalue='QUIT')
    wOptionMenu=widget_button(bar,value='OPTIONS',/menu)
        wOptionButton1=widget_button(wOptionMenu,value='TEXTURE',uvalue='TEXTURE')
        wOptionButton2=widget_button(wOptionMenu,value='MODE',/menu)
            wModeButton1=widget_button(wOptionButton2,value='ONLINE (default)',uvalue='ON')
            wModeButton2=widget_button(wOptionButton2,value='OFFLINE',uvalue='OFF')
        wOptionButton3=widget_button(wOptionMenu,value='BLACK/WHITE',uvalue='INVERT')
    wHelpMenu=widget_button(bar,value='HELP',/menu)
        wHelpButton1=widget_button(wHelpMenu,value='HELP',uvalue='HELP')
        wHelpButton2=widget_button(wHelpMenu,value='EXAMPLES',/menu)
            wConeButton=widget_button(wHelpButton2,value='CONE',/menu)
               if nf1 ne 0 then begin
                  for i=0,nf1-1 do begin
                   examplenames='cone '+strcompress(string(i+1),/remove_all)
                   wExampleButtons=widget_button(wConeButton,value=examplenames,uvalue='EXAMPLE')
                  endfor
               endif
            wellipsoidButton=widget_button(wHelpButton2,value='ELLIPSOID',/menu)
               if nf2 ne 0 then begin
                  for i=0,nf2-1 do begin
                   examplenames='ellipsoid '+strcompress(string(i+1),/remove_all)
                   wExampleButtons=widget_button(wellipsoidButton,value=examplenames,uvalue='EXAMPLE')
                  endfor
               endif
            wSpiralButton=widget_button(wHelpButton2,value='SPIRAL',/menu)
               if nf3 ne 0 then begin
                  for i=0,nf3-1 do begin
                   examplenames='spiral '+strcompress(string(i+1),/remove_all)
                   wExampleButtons=widget_button(wSpiralButton,value=examplenames,uvalue='EXAMPLE')
                  endfor
               endif
            wCylinderButton=widget_button(wHelpButton2,value='CYLINDER',/menu)
               if nf4 ne 0 then begin
                  for i=0,nf4-1 do begin
                   examplenames='cylinder '+strcompress(string(i+1),/remove_all)
                   wExampleButtons=widget_button(wCylinderButton,value=examplenames,uvalue='EXAMPLE')
                  endfor
               endif
            ;wWriteButton=widget_button(wHelpButton2,value='save example',uvalue='WRITE') ;saving examples
    wInnerBase=widget_base(wTopBase)

widget_control,wTopBase,/realize
widget_control,wFileButton2,sensitive=0
widget_control,wOptionButton3,sensitive=0
widget_control,wOptionButton2,sensitive=0
widget_control,wOptionButton1,sensitive=0

;--------------------State------------------------
State={wTopBase:wTopBase,$                  ;hlavni baze
       wInnerBase:wInnerBase,$              ;baze pod hlavni bazi
       wParamBase:0L,$                      ;baze s parametry
       wLeftBase:0L,$                       ;leva baze na wInnerBase (baze s objektem)
       wDraw:0L,$                           ;okno do ktereho se zobrazuje animace
       wRightBase:0L,$                      ;prava baze na wInnerBase (baze s budoucim spektrem)
       wRightDraw:0L,$                      ;okno do ktereho se zobrazuje spektrum
       wToggleButton1:0L,$                  ;centralni cara v pravem okne
       wParamSlider:0L,$                    ;jemnost deleni objemu
       wGenerateButton:0L,$                 ;generovat spektrum
       wWaitBase:0L,$                       ;hlaseni pri dlouhem vypoctu
       wSaveLabel:0L,$                      ;pocet framu pri ukladani
       wPosLabel:0L,$                       ;pozice v pravem okne
       wAnglesLabel:0L,$                    ;uhel urcujici nakloneni objektu
       wHelpBase:0L,$                       ;okno s napovedou
       wFileButton2:wFileButton2,$          ;ulozeni animace
       wConeButton:wConeButton,$            ;example menu
       wSpiralButton:wSpiralButton,$        ;example menu
       wCylinderButton:wCylinderButton,$    ;example menu
       wellipsoidButton:wellipsoidButton,$  ;example menu
       wOptionButton3:wOptionButton3,$      ;barva pozadi
       wOptionButton2:wOptionButton2,$      ;mod vykreslovani spektra
       wOptionButton1:wOptionButton1,$      ;vyber textury
       wExampleBase:0L,$                    ;okno pro posun prikladu
       wTextureLabel:0L,$                   ;label na wTextureBase obsahujici cestu zvolene textury
       wObjectLabel:0L,$                    ;label na wObjectBase obsahujici typ zvoleneho objektu
       wParamSlider1:0L,$                   ;parametr objektu nebo spektra
       wParamSlider2:0L,$                   ;parametr objektu nebo spektra
       wParamSlider3:0L,$                   ;parametr objektu nebo spektra
       wParamSlider4:0L,$                   ;parametr objektu nebo spektra
       wParamSlider5:0L,$                   ;parametr objektu nebo spektra
       wParamSlider6:0L,$                   ;parametr objektu nebo spektra
       wParamSlider7:0L,$                   ;parametr objektu nebo spektra
       wParamSlider8:0L,$                   ;parametr objektu nebo spektra
       wParamSlider9:0L,$                   ;parametr objektu nebo spektra
       wParamSlider10:0L,$                  ;parametr objektu nebo spektra
       wParamSlider11:0L,$                  ;parametr objektu nebo spektra
       wParamSlider12:0L,$                  ;parametr objektu nebo spektra
       wParamSlider13:0L,$                  ;parametr objektu nebo spektra
       wParamSlider14:0L,$                  ;parametr objektu nebo spektra
       oMpeg:obj_new(),$                    ;objekt mpg
       oHolder:obj_new(),$                  ;holder vsech objektu krome oWindow
       oImage:obj_new(),$                   ;textura
       oImagespiral:obj_new(),$             ;textura pro spiralu
       oObject:obj_new(),$                  ;objekt
       oBottom:obj_new(),$                  ;bottom
       oBottom2:obj_new(),$                 ;druhy bottom u valce
       oInner:obj_new(),$                   ;vnitrni objekt
       oLight:obj_new(),$                   ;osvetleni objektu
       oModel:obj_new(),$                   ;model spojujici objekt, bottomy a svetla
       oView:obj_new(),$                    ;scena
       oWindow:obj_new(),$                  ;leve okno
       oTrack:obj_new(),$                   ;trackball
       oSlit1:obj_new(),$                   ;sterbina
       oSlit2:obj_new(),$                   ;sterbina
       ParamExist:0,$                       ;existence baze s parametry
       RightDraw:0,$                        ;hodnota praveho okna
       DataPath:datapath,$                  ;cesta na data
       MainPath:mainpath,$                  ;cesta na hlavni adresar
       TexturePath:TexturePath,$            ;pole cest na obrazky textur
       ActualTexture:TexturePath(0),$       ;vybrana textura, ktera se zobrazuje pres objekt
       dimTexture:xTop/6,$                  ;velikost okna pro zobrazeni textur
       ActualObject:'cone',$                ;vybrany objekt
       ObjectPath:ObjectPath,$              ;pole cest na obrazky objektu
       dimObject:xTop/6,$                   ;velikost okna pro zobrazeni objektu
       RotAxis:[0.0,0.0,1.0],$              ;osa rotace kolem ktere se otaci objekt
       RunAnim:0,$                          ;1 pokud ma bezet animace
       xdim:xdim,$                          ;x velikost okna pro animaci
       ydim:ydim,$                          ;y velikost okno pro animaci
       xTop:xTop,$                          ;x velikost hlavniho okna
       yTop:yTop,$                          ;y velikost hlavniho okna
       SaveAnim:0,$                         ;cislo framu pri ukladani animace
       SaveType:' ',$                       ;typ souboru pro ukladani animace
       Timer:0.01,$                         ;cas za ktery se ma vygenerovat dalsi udalost v animaci
       dTime:1.0,$                          ;narust casu pri kazdem kroku animace
       density:10,$                         ;parametr kvality zobrazeni objektu
       btndown:0B,$                         ;kliknuti mysi v levem okne
       up:1.0,$                             ;horni mez definujici oblast obrazku z ktere se tvori textura
       down:0.5,$                           ;dolni mez
       up_bottom:1.0,$                      ;horni mez pro bottom
       down_bottom:0.5,$                    ;dolni mez pro bottom
       ParamRot:0L,$                        ;rychlost rotace
       ParamTex:0L,$                        ;rychlost pohybu nahoru po objektu
       Radius:0.5,$                         ;polomer valce a kuzele
       Height:0.8,$                         ;vyska valce, kuzele a spiraly
       A:0.5,$                              ;hlavni poloosa elipsoidu
       Exc:0.0,$                            ;excentricita elipsoidu
       Width:0.03,$                         ;polotloustka spiraly
       R:0.2,$                              ;polomer spiraly
       Dist:0.2,$                           ;parametr roztazeni spiraly
       V:0,$                                ;rychlost pohybu po spirale
       SlitPos:-1.0,$                       ;pozice sterbiny (od -1 do 1)
       SlitWid:0.01,$                       ;sirka sterbiny
       LineWidth:200,$                      ;sirka spektralni cary
       Levels:100,$                         ;parametr urcujici jemnost deleni spiraly pri vypoctu pruseciku
       Accuracy:-3,$                        ;parametr presnosti vypoctu pruseciku se spiralou
       Distribution:0.0,$                   ;parametr urcujici rozlozeni hmoty
       Step:0.05,$                          ;parametr urcujici jemnost integrace pres objekt
       Nstep:100,$                          ;parametr urcujici jemnost integrace pres sirku sterbiny
       Mode:1,$                             ;zpusob vykreslovani spektra
       Spectra:dblarr(xdim,ydim),$          ;spektrum
       Profile:dblarr(xdim),$               ;vybrany profil spektra
       Negativ:0,$                          ;urcuje barvu pozadi
       Nfiles:[nf1,nf2,nf3,nf4],$           ;pocet souboru s priklady
       File:0,$                             ;aktualni poradi souboru s prikladem
       ExampleObject:'cone',$               ;objekt v prikladu
       ParamOffset:[0.0,0.0],$              ;poloha ParamBase
       ExamOffset:[0.0,0.0],$               ;poloha ExampleBase
       Matrix:[[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0]]}
                                            ;transformacni matice mezi soustavou pozorovatele a objektu
widget_control,wTopBase,set_uvalue=State
xmanager,'Animate',wTopBase,cleanup='Animate_cleanup'

end