FUNCTION ellipsoid::Init, POS=pos, HEIGHT=height, A=a, EXC=exc, DENSITY=density, TEX_COORDS=tex_coords, _EXTRA=e
    IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0
    self.height = 0.5
    self.a = 0.5
    self.exc = 0.5
    self.density = 10
    self.pos = [0.0,0.0,0.0]
    IF (N_ELEMENTS(height) EQ 1) THEN self.height = height
    IF (N_ELEMENTS(a) EQ 1) THEN self.a = a
    IF (N_ELEMENTS(exc) EQ 1) THEN self.exc = exc
    IF (N_ELEMENTS(density) EQ 1) THEN self.density = density
    IF (N_ELEMENTS(pos) EQ 3) THEN self.pos = pos
    self.oPoly = OBJ_NEW('IDLgrPolygon', SHADING=1, _EXTRA=e)
    self->Add,self.oPoly
    self->BuildPoly
    RETURN, 1
END

;----------------------------------------------------------------------------
PRO ellipsoid::Cleanup
    OBJ_DESTROY, self.oPoly
    self->IDLgrModel::Cleanup
END

;----------------------------------------------------------------------------
PRO ellipsoid::SetProperty, POS=pos, HEIGHT=height, A=a, EXC=exc, DENSITY=density, _EXTRA=e
    self->IDLgrModel::SetProperty, _EXTRA=e
    self.oPoly->SetProperty, _EXTRA=e
    IF (N_ELEMENTS(pos) EQ 3) THEN self.pos = pos
    IF (N_ELEMENTS(height) EQ 1) THEN self.height=height
    IF (N_ELEMENTS(a) EQ 1) THEN self.a=a
    IF (N_ELEMENTS(exc) EQ 1) THEN self.exc=exc
    IF (N_ELEMENTS(density) EQ 1) THEN self.density = density
    self->BuildPoly
END

;----------------------------------------------------------------------------
PRO ellipsoid::GetProperty, POS=pos, HEIGHT=height, A=a, EXC=exc, DENSITY=density,$
                         POBJ=pobj, _EXTRA=e
    self->IDLgrModel::GetProperty, _EXTRA=e
    self.oPoly->GetProperty, _EXTRA=e
    pos = self.pos
    height = self.height
    a=self.a
    exc=self.exc
    density = self.density
    pobj = self.oPoly
END

;----------------------------------------------------------------------------
PRO ellipsoid::BuildPoly

    ;---------------data-----------------
    if self.exc eq 1.0 then self.exc=0.99
    h=self.a
    r02=h*(1-self.exc^2)*(2*self.a-h)
    theta0=acos((h-self.a)/sqrt((h-self.a)^2+r02))
    theta=theta0+(!pi-theta0)*findgen(self.density)/(self.density-1)
    fi=2*!pi*findgen(4*(self.density-1))/(4*(self.density-1))
    nfi=n_elements(fi)
    ntheta=n_elements(theta)
    n=(ntheta-1)*nfi+1
    x=findgen(n)
    y=findgen(n)
    z=findgen(n)

    for i=0,ntheta-2 do begin
        zcoord=self.a+(2*self.a*cos(theta(i))*sqrt((1-self.exc^2)*(1-self.exc^2*cos(theta(i))^2)))/(2*(1-self.exc^2*cos(theta(i))^2))
        r=sqrt(zcoord*(1-self.exc^2)*(2*self.a-zcoord))
        for j=0,nfi-1 do begin
            x(j+i*nfi)=r*cos(fi(j))+self.pos(0)
            y(j+i*nfi)=r*sin(fi(j))+self.pos(1)
            z(j+i*nfi)=zcoord+self.pos(2)
        endfor
    endfor

    x(n-1)=0.0+self.pos(0)
    y(n-1)=0.0+self.pos(1)
    z(n-1)=0.0+self.pos(2)

    data=transpose([[x],[y],[z]])

    ;--------------polygons----------------------------
    dim=nfi*(ntheta-2)
    polygon1=intarr(5,dim)
    polygon1(0,*)=4
    polygon1(1,*)=indgen(dim)
    polygon1(2,*)=indgen(dim)+1
    polygon1(3,*)=indgen(dim)+nfi+1
    polygon1(4,*)=indgen(dim)+nfi
    for i=0,ntheta-3 do begin
        polygon1(2,(i+1)*nfi-1)=polygon1(1,i*nfi)
        polygon1(3,(i+1)*nfi-1)=polygon1(4,i*nfi)
    endfor
    polygon1=reform(polygon1,5*dim)

    polygon2=intarr(4,nfi)
    polygon2(0,*)=3
    polygon2(1,*)=indgen(nfi)+dim
    polygon2(2,*)=indgen(nfi)+dim+1
    polygon2(3,*)=n-1
    polygon2(2,nfi-1)=polygon2(1,0)
    polygon2=reform(polygon2,4*nfi)

    polygons=[polygon1,polygon2]

    self.oPoly->SetProperty, DATA=data, POLYGONS=polygons

END

;-----------------------------------------------------------
PRO ellipsoid_define
    struct = { ellipsoid, $
               INHERITS IDLgrModel, $
               pos: [0.0,0.0,0.0], $
               height: 0.5, $
               a: 0.5, $
               exc: 0.5, $
               density: 4, $
               oPoly: OBJ_NEW() $
             }
END







